#include "stdafx.h"

#include <shellapi.h>
#include <strsafe.h>
#pragma comment(lib,"strsafe.lib")//safe strcopy

#include "resource.h"

#include "menu.h"

#ifdef _DEBUG
#define DBGTRACE(x) OutputDebugString(_T(x))
#else
#define DBGTRACE(x)
#endif

// Global Variables:
WNDCLASSEX wcex; //reuse wcex.hInstance and wcex.hIcon
#define DPS_WNDCLASS_NAME  _T("DisplayPowerSaverWndClass")
HMENU htrayMenu;
NOTIFYICONDATA  trayicon;
#define WM_TRAY_NOTIFY (WM_APP+11)
#define DPS_ATOM_NAME _T("DisplayPowerSaverAtom")


//message-only window proc
LRESULT CALLBACK dpsWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_COMMAND:
		switch (LOWORD(wParam))// Parse menu selections
		{
			case IDM_TURN_DISPLAY_OFF:
				DBGTRACE("OnDisplayOff\n");
				return ::DefWindowProc(hWnd, WM_SYSCOMMAND, SC_MONITORPOWER, 2);
			break;
			case IDM_START_SCREENSAVER:
				DBGTRACE("OnScreensaver\n");
				return ::DefWindowProc(hWnd, WM_SYSCOMMAND, SC_SCREENSAVE, 0);
			break;
			case IDM_EXIT:
				DBGTRACE("IDM_EXIT\n");
				if (IDYES==::MessageBox(HWND_DESKTOP, _T("Do you wish to exit application?"),
						_T("Display Power Saver"), MB_YESNO | MB_SYSTEMMODAL | MB_ICONQUESTION | MB_DEFBUTTON2))
					DestroyWindow(hWnd);
				SetProcessWorkingSetSize(GetCurrentProcess(),(SIZE_T)-1,(SIZE_T)-1);
			break;
		SetProcessWorkingSetSize(GetCurrentProcess(),(SIZE_T)-1,(SIZE_T)-1);
		}
	break;

	case WM_DDE_INITIATE:
		DBGTRACE("WM_DDE_INITIATE received\n");
		DestroyWindow(hWnd);
	break;

	case WM_TRAY_NOTIFY:
		if (LOWORD(lParam)==WM_RBUTTONUP)
		{
			DBGTRACE("WM_RBUTTONUP\n");
			POINT mousepos;
			if (!GetCursorPos(&mousepos)) return 0;
			//MSDN Q135788 fix
			::SetForegroundWindow(hWnd);  
			::TrackPopupMenuEx(GetSubMenu(htrayMenu,0),TPM_RIGHTALIGN,mousepos.x, mousepos.y, hWnd, NULL);
			::PostMessage(hWnd, WM_NULL, 0, 0);
			SetProcessWorkingSetSize(GetCurrentProcess(),(SIZE_T)-1,(SIZE_T)-1);
		}
	break;

	case WM_DESTROY:
		PostQuitMessage(0);
	break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
return 0;
}


//main
#ifdef _ATL_MIN_CRT //release build
int WINAPI WinMainCRTStartup(void)//no crt
#else
int APIENTRY _tWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
#endif
{
	//init structs
	SecureZeroMemory(&wcex, sizeof(WNDCLASSEX));
	SecureZeroMemory(&trayicon, sizeof(NOTIFYICONDATA));

	wcex.cbSize			= sizeof(WNDCLASSEX);
	wcex.hInstance		= GetModuleHandle(NULL);//instead of hInstance for no crt
	wcex.lpfnWndProc	= dpsWndProc;
	wcex.lpszMenuName	= MAKEINTRESOURCE(IDC_DISPLAYPOWERSAVER);
	wcex.lpszClassName	= DPS_WNDCLASS_NAME;
	wcex.hIconSm = (HICON)::LoadImage(wcex.hInstance, MAKEINTRESOURCE(IDI_DISPLAYPOWERSAVER),
					IMAGE_ICON, ::GetSystemMetrics(SM_CXSMICON), ::GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);

	if (!RegisterClassEx(&wcex)) return 0;


	//find previous instance
	HWND hprev=FindWindowEx(HWND_MESSAGE, NULL, DPS_WNDCLASS_NAME, NULL);

	//create message-only window
	trayicon.hWnd = CreateWindow(wcex.lpszClassName, NULL, WS_OVERLAPPEDWINDOW, CW_USEDEFAULT,
									0, CW_USEDEFAULT, 0, HWND_MESSAGE, NULL, wcex.hInstance, NULL);
	if (!trayicon.hWnd) return FALSE;

	//signal previous instance to close
	ATOM dpsAtom=GlobalAddAtom(DPS_ATOM_NAME);
	if (hprev) SendMessage(hprev, WM_DDE_INITIATE, (WPARAM)trayicon.hWnd, MAKELPARAM(dpsAtom,0));
	GlobalDeleteAtom(dpsAtom);


	//set menu bitmaps
	htrayMenu=::GetMenu(trayicon.hWnd);

//make DPI-aware?
	HBITMAP hbmp1=LoadBitmap(wcex.hInstance, MAKEINTRESOURCE(IDB_BMPOFF));
	if (hbmp1) SetMenuItemBitmaps(htrayMenu, IDM_TURN_DISPLAY_OFF, MF_BYCOMMAND, hbmp1, hbmp1);


	HBITMAP hbmp2=(HBITMAP)LoadImage(wcex.hInstance, MAKEINTRESOURCE(IDB_BMPSCR),
												IMAGE_BITMAP, 0,0, LR_CREATEDIBSECTION);
	if (hbmp2)
		if (ReplaceDIBColor(&hbmp2, RGB(255,0,255), GetSysColor(COLOR_MENU)))
			SetMenuItemBitmaps(htrayMenu, IDM_START_SCREENSAVER, MF_BYCOMMAND, hbmp2, hbmp2);

	HBITMAP hbmp3=LoadBitmap(wcex.hInstance, MAKEINTRESOURCE(IDB_BMPEXIT));
	if (hbmp3) SetMenuItemBitmaps(htrayMenu, IDM_EXIT, MF_BYCOMMAND, hbmp3, hbmp3);

	
	//fill tray icon info
	trayicon.cbSize=sizeof(NOTIFYICONDATA);
	trayicon.uFlags=(NIF_ICON | NIF_MESSAGE | NIF_TIP);
	//trayicon.hWnd already set
	trayicon.hIcon=wcex.hIconSm;
	trayicon.dwInfoFlags=NIIF_USER;
	trayicon.uCallbackMessage=WM_TRAY_NOTIFY;
	trayicon.uTimeout=3000;//ms
	StringCchCopy(trayicon.szTip, sizeof(trayicon.szTip), _T("Display Power Saver 2.8.2\nRight-click for options.\0"));

	//add tray icon
	Shell_NotifyIcon(NIM_ADD, &trayicon);

	//minimize memory usage
	SetProcessWorkingSetSize(GetCurrentProcess(),(SIZE_T)-1,(SIZE_T)-1);

	// Main message loop:
	MSG msg;
	while (GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}


	//cleanup
	Shell_NotifyIcon(NIM_DELETE, &trayicon);
	DeleteObject(hbmp1);
	DeleteObject(hbmp2);
	DeleteObject(hbmp3);	

#ifdef _ATL_MIN_CRT
	ExitProcess(0);
#endif
return 0;
}
